referrerpolicy=no-referrer-when-downgrade

Enum Call

pub enum Call<T: Config<I>, I: 'static = ()> {
    fund_bounty {
        asset_kind: Box<T::AssetKind>,
        value: T::Balance,
        curator: AccountIdLookupOf<T>,
        metadata: T::Hash,
    },
    fund_child_bounty {
        parent_bounty_id: BountyIndex,
        value: T::Balance,
        metadata: T::Hash,
        curator: Option<AccountIdLookupOf<T>>,
    },
    propose_curator {
        parent_bounty_id: BountyIndex,
        child_bounty_id: Option<BountyIndex>,
        curator: AccountIdLookupOf<T>,
    },
    accept_curator {
        parent_bounty_id: BountyIndex,
        child_bounty_id: Option<BountyIndex>,
    },
    unassign_curator {
        parent_bounty_id: BountyIndex,
        child_bounty_id: Option<BountyIndex>,
    },
    award_bounty {
        parent_bounty_id: BountyIndex,
        child_bounty_id: Option<BountyIndex>,
        beneficiary: BeneficiaryLookupOf<T, I>,
    },
    close_bounty {
        parent_bounty_id: BountyIndex,
        child_bounty_id: Option<BountyIndex>,
    },
    check_status {
        parent_bounty_id: BountyIndex,
        child_bounty_id: Option<BountyIndex>,
    },
    retry_payment {
        parent_bounty_id: BountyIndex,
        child_bounty_id: Option<BountyIndex>,
    },
    // some variants omitted
}
Contains a variant per dispatchable extrinsic that this pallet has.

Variants



fund_bounty

Fund a new bounty with a proposed curator, initiating the payment from the funding source to the bounty account/location.

Dispatch Origin

Must be Config::SpendOrigin with the Success value being at least the bounty value converted to native balance using Config::BalanceConverter. The converted native amount is validated against the maximum spendable amount returned by Config::SpendOrigin.

Details
  • The SpendOrigin must have sufficient permissions to fund the bounty.
  • The bounty value (in asset balance) is converted to native balance for validation.
  • In case of a funding failure, the bounty status must be updated with the check_status call before retrying with retry_payment call.
Parameters
  • asset_kind: An indicator of the specific asset class to be funded.
  • value: The total payment amount of this bounty.
  • curator: Address of bounty curator.
  • metadata: The hash of an on-chain stored preimage with bounty metadata.
Events

Emits Event::BountyCreated and Event::Paid if successful.

Fields

asset_kind: Box<T::AssetKind>
value: T::Balance
metadata: T::Hash


fund_child_bounty

Fund a new child-bounty with a proposed curator, initiating the payment from the parent bounty to the child-bounty account/location.

Dispatch Origin

Must be signed by the parent curator.

Details
  • If curator is not provided, the child-bounty will default to using the parent curator, allowing the parent curator to immediately call check_status and award_bounty to payout the child-bounty.
  • In case of a funding failure, the child-/bounty status must be updated with the check_status call before retrying with retry_payment call.
Parameters
  • parent_bounty_id: Index of parent bounty for which child-bounty is being added.
  • value: The payment amount of this child-bounty.
  • metadata: The hash of an on-chain stored preimage with child-bounty metadata.
  • curator: Address of child-bounty curator.
Events

Emits Event::ChildBountyCreated and Event::Paid if successful.

Fields

parent_bounty_id: BountyIndex
value: T::Balance
metadata: T::Hash


propose_curator

Propose a new curator for a child-/bounty after the previous was unassigned.

Dispatch Origin

Must be signed by T::SpendOrigin for a bounty, or by the parent bounty curator for a child-bounty.

Details
  • The child-/bounty must be in the CuratorUnassigned state.
  • For a bounty, the SpendOrigin must have sufficient permissions to propose the curator.
Parameters
  • parent_bounty_id: Index of bounty.
  • child_bounty_id: Index of child-bounty.
  • curator: Account to be proposed as the curator.
Events

Emits Event::CuratorProposed if successful.

Fields

parent_bounty_id: BountyIndex
child_bounty_id: Option<BountyIndex>


accept_curator

Accept the curator role for a child-/bounty.

Dispatch Origin

Must be signed by the proposed curator.

Details
  • The child-/bounty must be in the Funded state.
  • The curator must accept the role by calling this function.
  • A deposit will be reserved from the curator and refunded upon successful payout.
Parameters
  • parent_bounty_id: Index of parent bounty.
  • child_bounty_id: Index of child-bounty.
Events

Emits Event::BountyBecameActive if successful.

Fields

parent_bounty_id: BountyIndex
child_bounty_id: Option<BountyIndex>


unassign_curator

Unassign curator from a child-/bounty.

Dispatch Origin

This function can only be called by the RejectOrigin or the child-/bounty curator.

Details
  • If this function is called by the RejectOrigin, or by the parent curator in the case of a child bounty, we assume that the curator is malicious or inactive. As a result, we will slash the curator when possible.
  • If the origin is the child-/bounty curator, we take this as a sign they are unable to do their job and they willingly give up. We could slash them, but for now we allow them to recover their deposit and exit without issue. (We may want to change this if it is abused).
  • If successful, the child-/bounty status is updated to CuratorUnassigned. To reactivate the bounty, a new curator must be proposed and must accept the role.
Parameters
  • parent_bounty_id: Index of parent bounty.
  • child_bounty_id: Index of child-bounty.
Events

Emits Event::CuratorUnassigned if successful.

Fields

parent_bounty_id: BountyIndex
child_bounty_id: Option<BountyIndex>


award_bounty

Awards the child-/bounty to a beneficiary account/location, initiating the payout payments to both the beneficiary and the curator.

Dispatch Origin

This function can only be called by the RejectOrigin or the child-/bounty curator.

Details
  • The child-/bounty must be in the Active state.
  • if awarding a parent bounty it must not have active or funded child bounties.
  • Initiates payout payment from the child-/bounty to the beneficiary account/location.
  • If successful the child-/bounty status is updated to PayoutAttempted.
  • In case of a payout failure, the child-/bounty status must be updated with check_status call before retrying with retry_payment call.
Parameters
  • parent_bounty_id: Index of parent bounty.
  • child_bounty_id: Index of child-bounty.
  • beneficiary: Account/location to be awarded the child-/bounty.
Events

Emits Event::BountyAwarded and Event::Paid if successful.

Fields

parent_bounty_id: BountyIndex
child_bounty_id: Option<BountyIndex>
beneficiary: BeneficiaryLookupOf<T, I>


close_bounty

Cancel an active child-/bounty. A payment to send all the funds to the funding source is initialized.

Dispatch Origin

This function can only be called by the RejectOrigin or the parent bounty curator.

Details
  • If the child-/bounty is in the Funded state, a refund payment is initiated.
  • If the child-/bounty is in the Active state, a refund payment is initiated and the child-/bounty status is updated with the curator account/location.
  • If the child-/bounty is in the funding or payout phase, it cannot be canceled.
  • In case of a refund failure, the child-/bounty status must be updated with the check_status call before retrying with retry_payment call.
Parameters
  • parent_bounty_id: Index of parent bounty.
  • child_bounty_id: Index of child-bounty.
Events

Emits Event::BountyCanceled and Event::Paid if successful.

Fields

parent_bounty_id: BountyIndex
child_bounty_id: Option<BountyIndex>


check_status

Check and update the payment status of a child-/bounty.

Dispatch Origin

Must be signed.

Details
  • If the child-/bounty status is FundingAttempted, it checks if the funding payment has succeeded. If successful, the bounty status becomes Funded.
  • If the child-/bounty status is RefundAttempted, it checks if the refund payment has succeeded. If successful, the child-/bounty is removed from storage.
  • If the child-/bounty status is PayoutAttempted, it checks if the payout payment has succeeded. If successful, the child-/bounty is removed from storage.
Parameters
  • parent_bounty_id: Index of parent bounty.
  • child_bounty_id: Index of child-bounty.
Events

Emits Event::BountyBecameActive if the child/bounty status transitions to Active. Emits Event::BountyRefundProcessed if the refund payment has succeed. Emits Event::BountyPayoutProcessed if the payout payment has succeed. Emits Event::PaymentFailed if the funding, refund our payment payment has failed.

Fields

parent_bounty_id: BountyIndex
child_bounty_id: Option<BountyIndex>


retry_payment

Retry the funding, refund or payout payments.

Dispatch Origin

Must be signed.

Details
  • If the child-/bounty status is FundingAttempted, it retries the funding payment from funding source the child-/bounty account/location.
  • If the child-/bounty status is RefundAttempted, it retries the refund payment from the child-/bounty account/location to the funding source.
  • If the child-/bounty status is PayoutAttempted, it retries the payout payment from the child-/bounty account/location to the beneficiary account/location.
Parameters
  • parent_bounty_id: Index of parent bounty.
  • child_bounty_id: Index of child-bounty.
Events

Emits Event::Paid if the funding, refund or payout payment has initiated.

Fields

parent_bounty_id: BountyIndex
child_bounty_id: Option<BountyIndex>

Implementations

Source

impl<T: Config<I>, I: 'static> Call<T, I>

pub fn new_call_variant_fund_bounty( asset_kind: Box<T::AssetKind>, value: T::Balance, curator: AccountIdLookupOf<T>, metadata: T::Hash, ) -> Self

Create a call with the variant fund_bounty.

pub fn new_call_variant_fund_child_bounty( parent_bounty_id: BountyIndex, value: T::Balance, metadata: T::Hash, curator: Option<AccountIdLookupOf<T>>, ) -> Self

Create a call with the variant fund_child_bounty.

pub fn new_call_variant_propose_curator( parent_bounty_id: BountyIndex, child_bounty_id: Option<BountyIndex>, curator: AccountIdLookupOf<T>, ) -> Self

Create a call with the variant propose_curator.

pub fn new_call_variant_accept_curator( parent_bounty_id: BountyIndex, child_bounty_id: Option<BountyIndex>, ) -> Self

Create a call with the variant accept_curator.

pub fn new_call_variant_unassign_curator( parent_bounty_id: BountyIndex, child_bounty_id: Option<BountyIndex>, ) -> Self

Create a call with the variant unassign_curator.

pub fn new_call_variant_award_bounty( parent_bounty_id: BountyIndex, child_bounty_id: Option<BountyIndex>, beneficiary: BeneficiaryLookupOf<T, I>, ) -> Self

Create a call with the variant award_bounty.

pub fn new_call_variant_close_bounty( parent_bounty_id: BountyIndex, child_bounty_id: Option<BountyIndex>, ) -> Self

Create a call with the variant close_bounty.

pub fn new_call_variant_check_status( parent_bounty_id: BountyIndex, child_bounty_id: Option<BountyIndex>, ) -> Self

Create a call with the variant check_status.

pub fn new_call_variant_retry_payment( parent_bounty_id: BountyIndex, child_bounty_id: Option<BountyIndex>, ) -> Self

Create a call with the variant retry_payment.